import { Component, OnInit } from '@angular/core';
import {FormBuilder, FormControl, FormGroup, Validators} from "@angular/forms";
import {MatSnackBar} from "@angular/material/snack-bar";
import {TaskService} from "../task.service";
import {Router} from "@angular/router";

@Component({
  selector: 'app-task-add',
  templateUrl: './task-add.component.html',
  styleUrls: ['./task-add.component.css']
})
export class TaskAddComponent implements OnInit {
  public options: FormGroup;

  constructor(fb: FormBuilder, private snackBar: MatSnackBar, private ts: TaskService, private router: Router) {
    this.options = fb.group({
      hideRequired: false,
      floatLabel: 'auto'
    })
  }

  form = new FormGroup({
    name: new FormControl('', Validators.required),
    description: new FormControl('', Validators.required),
    state: new FormControl('', Validators.required),
    date: new FormControl((new Date()).toISOString(), Validators.required,)
  });

  states = ["DONE", "DOING", "TODO"]
  submitted = false;
  submit(){
    this.submitted = true;
    if (!this.form.invalid) {
      this.ts.addTask(this.form.value.name, this.form.value.state, this.form.value.description, (new Date(this.form.value.date)).getTime().toString()).then(() => this.router.navigate(['/']))
    }
  }

  ngOnInit(): void {
  }

}
