import {Component, OnInit} from '@angular/core';
import {TaskService} from "../task.service";
import {Task} from "../task.model";
import {CdkDragDrop, CdkDropList, moveItemInArray, transferArrayItem} from "@angular/cdk/drag-drop";

@Component({
  selector: 'app-task-holder',
  templateUrl: './task-holder.component.html',
  styleUrls: ['./task-holder.component.css']
})
export class TaskHolderComponent implements OnInit{
  done: any = [];
  doing: any = [];
  todo: any = [];

  constructor(private ts: TaskService) {}

  ngOnInit():void{
    this.getData()
  }

  demoData(){
    this.ts.generateDemoData().then(() => this.getData()).catch(err => alert(err));

  }

  /**
   * Current item ordering will be saved. This is done by rearranging the ids. First index will get the lowest id and the last index the highest id
   * Example: [{id: 11, name: 1}, {id: 2, name: 2}, {id: 13, name: 3}] --> [{id: 2, name: 1}, {id: 11, name: 2}, {id: 13, name: 3}]
   *
   * When ordering is finished, data will be saved to DB and data will be refreshed
   * @param container Current Container to doOrdering
   */
  doOrdering(container:  CdkDropList<[]>){
    let result1 = [] as Task[];
    (result1 as Task[]) = container.data;
    let ids: number[] = [];
    result1.forEach(r => {

      ids.push(parseInt(r.id))

    })
    ids.sort(function(a, b) {
      return a - b;
    });
    let count = 0;
    result1.forEach(r => {
      r.id = ids[count] + "";
      count++;
    });
    // Change state enum after list change
    // 0 = done
    // 1 = doing
    // 2 = todo
    if (container.id == "cdk-drop-list-0") {
      this.done = result1 as [];
    } else if (container.id == "cdk-drop-list-1") {
      this.doing = result1 as [];
    } else if (container.id == "cdk-drop-list-2") {
      this.todo = result1 as [];
    }
    result1.forEach(r1 => {
      const newDate = new Date(r1.date);
      this.ts.editTask(r1.id, r1.name, r1.state, r1.description, newDate.getTime().toString()).catch(err => alert(err)).then(() => this.getData());
    })
  }

  /**
   * Transferring item state from DONE, DOING and to do
   * When transferring is finished, items will be ordered by doOrdering and also be saved to DB
   * @param event Current drag and drop event
   */
  doTransfer(event: CdkDragDrop<[]>){
    // Change state enum after list change
    // 0 = done
    // 1 = doing
    // 2 = todo
    let result = {} as Task;
    if (event.container.id == "cdk-drop-list-0") {
      (result as Task) = event.container.data[event.currentIndex];
      result.state = "DONE"
      this.done[event.currentIndex] = (result as never);
    } else if (event.container.id == "cdk-drop-list-1") {
      (result as Task) = event.container.data[event.currentIndex];
      result.state = "DOING"
      this.doing[event.currentIndex] = (result as never);
    } else if (event.container.id == "cdk-drop-list-2") {
      (result as Task) = event.container.data[event.currentIndex];
      result.state = "TODO"
      this.todo[event.currentIndex] = (result as never);
    }
    const newDate = new Date(result.date);
    this.ts.editTask(result.id, result.name, result.state, result.description, newDate.getTime().toString()).catch(err => alert(err)).then(() => this.doOrdering(event.container));

  }

  drop(event: CdkDragDrop<[]>) {
    if (event.previousContainer === event.container) {
      moveItemInArray(event.container.data, event.previousIndex, event.currentIndex);
      this.doOrdering(event.container)
    } else {
      transferArrayItem(event.previousContainer.data,
        event.container.data,
        event.previousIndex,
        event.currentIndex);
      this.doTransfer(event);
    }
  }

  getData(){
    this.ts.filteredTodos("done").then(res => {
      this.done = res.data;
    }).catch(err => console.error(err))
    this.ts.filteredTodos("doing").then(res => {
      this.doing = res.data;
    }).catch(err => console.error(err))
    this.ts.filteredTodos("todo").then(res => {
      this.todo = res.data;
    }).catch(err => console.error(err))
  }

  deleteTask(id: string, list: string) {
    switch (list) {
      case "todo":
        this.todo = this.filterDelete(this.todo, id)
        break
      case "doing":
        this.doing = this.filterDelete(this.doing, id)
        break
      case "done":
        this.done = this.filterDelete(this.done, id)
        break
    }
  }

  filterDelete(list: any, id: string): any{
    return list.filter((i: any) => i.id != id)
  }
}
